import logging
from collections import namedtuple
from typing import Optional

import cyvcf2

from loqusdb.constants import CHROM_TO_INT, GENOTYPE_MAP, GRCH37, PAR
from loqusdb.models import Case, Variant

LOG = logging.getLogger(__name__)

Position = namedtuple("Position", "chrom pos")


# These are coordinate for the pseudo autosomal regions in GRCh37


def check_par(chrom, pos, genome_build):
    """Check if a coordinate is in the PAR region

    Args:
        chrom(str)
        pos(int)
        genome_build(str): Genome build. Ex. GRCh37 or GRCh38

    Returns:
        par(bool)
    """
    return any(
        pos >= interval[0] and pos <= interval[1] for interval in PAR[genome_build].get(chrom, [])
    )


def get_variant_id(variant, keep_chr_prefix=None):
    """Get a variant id on the format chrom_pos_ref_alt

    Args:
        variant (cyvcf2.Variant)
        keep_chr_prefix(bool): Retain chr/CHR/Chr prefix when present

    Returns:
        variant (models.ProfileVariant)
    """

    chrom = variant.CHROM
    if not keep_chr_prefix:
        if chrom.lower().startswith("chr"):
            chrom = chrom[3:]
    return "_".join([str(chrom), str(variant.POS), str(variant.REF), str(variant.ALT[0])])


def is_greater(a, b, genome_build):
    """Check if position a is greater than position b
    This will look at chromosome and position.

    For example a position where chrom = 2 and pos = 300 is greater than a position where
    chrom = 1 and pos = 1000

    If any of the chromosomes is outside [1-22,X,Y,MT] or [chr1-chr22,chrX,chrY,chrM] we can not say which is biggest.

    Args:
        a,b(Position)
        genome_build(str): Genome build. Ex. GRCh37 or GRCh38

    Returns:
        bool: True if a is greater than b
    """

    a_chrom = CHROM_TO_INT[genome_build].get(a.chrom, 0)
    b_chrom = CHROM_TO_INT[genome_build].get(b.chrom, 0)

    if a_chrom == 0 or b_chrom == 0:
        return False

    if a_chrom > b_chrom:
        return True

    return a_chrom == b_chrom and a.pos > b.pos


def get_coords(variant, keep_chr_prefix, genome_build):
    """Returns a dictionary with position information

    Args:
        variant(cyvcf2.Variant)
        keep_chr_prefix(bool): Retain chr/CHR/Chr prefix when present
        genome_build(str): Genome build. Ex. GRCh37 or GRCh38

    Returns:
        coordinates(dict)
    """
    coordinates = {
        "chrom": None,
        "end_chrom": None,
        "sv_length": None,
        "sv_type": None,
        "pos": None,
        "end": None,
    }
    chrom = variant.CHROM
    if not keep_chr_prefix:
        if chrom.startswith(("chr", "CHR", "Chr")):
            chrom = chrom[3:]
    coordinates["chrom"] = chrom
    end_chrom = chrom

    pos = int(variant.POS)
    alt = variant.ALT[0]

    # Get the end position
    # This will be None for non-svs
    end_pos = variant.INFO.get("END")
    end = int(end_pos) if end_pos else int(variant.end)
    coordinates["end"] = end

    sv_type = variant.INFO.get("SVTYPE")
    length = variant.INFO.get("SVLEN")
    sv_len = abs(length) if length else end - pos
    # Translocations will sometimes have a end chrom that differs from chrom
    if sv_type == "BND":
        other_coordinates = alt.strip("ATCGN").strip("[]").split(":")
        end_chrom = other_coordinates[0]
        if not keep_chr_prefix:
            if end_chrom.startswith(("chr", "CHR", "Chr")):
                end_chrom = end_chrom[3:]

        end = int(other_coordinates[1])

        # Set 'infinity' to length if translocation
        sv_len = float("inf")

    # Insertions often have length 0 in VCF
    if sv_len == 0 and alt != "<INS>":
        sv_len = len(alt)

    if (pos == end) and (sv_len > 0) and sv_len != float("inf"):
        end = pos + sv_len

    position = Position(chrom, pos)
    end_position = Position(end_chrom, end)

    # If 'start' is greater than 'end', switch positions
    if is_greater(position, end_position, genome_build=genome_build):
        end_chrom = position.chrom
        end = position.pos

        chrom = end_position.chrom
        pos = end_position.pos

    coordinates["end_chrom"] = end_chrom
    coordinates["pos"] = pos
    coordinates["end"] = end
    coordinates["sv_length"] = sv_len
    coordinates["sv_type"] = sv_type

    return coordinates


def build_variant(
    variant: cyvcf2.Variant,
    case_obj: Case,
    case_id: Optional[str] = None,
    gq_threshold: Optional[int] = None,
    gq_qual: Optional[bool] = False,
    keep_chr_prefix: Optional[bool] = False,
    ignore_gq_if_unset: Optional[bool] = False,
    genome_build: Optional[str] = None,
) -> Variant:
    """Return a Variant object

    Take a cyvcf2 formated variant line and return a models.Variant.

    If criteria are not fulfilled, eg variant has no GT call or quality.
    is below gq threshold then return None.


    Args:
        variant(cyvcf2.Variant)
        case_obj(Case): We need the case object to check individuals sex
        case_id(str): The case id
        gq_threshold(int): Genotype Quality threshold
        gq_qual(bool): Use variant.QUAL for quality instead of GQ
        keep_chr_prefix(bool): Retain chr/CHR/Chr prefix when present
        ignore_gq_if_unset(bool): Ignore GQ threshold check for variants that do not have GQ or QUAL set.
        genome_build(str): Genome build. Ex. GRCh37 or GRCh38

    Return:
        formated_variant(models.Variant): A variant dictionary
    """
    variant_obj = None

    sv = False
    # Let cyvcf2 tell if it is a Structural Variant or not
    if variant.var_type == "sv":
        sv = True

    # chrom_pos_ref_alt
    variant_id = get_variant_id(variant, keep_chr_prefix)

    ref = variant.REF
    # ALT is an array in cyvcf2
    # We allways assume splitted and normalized VCFs
    alt = variant.ALT[0]

    coordinates = get_coords(variant, keep_chr_prefix, genome_build=genome_build)
    chrom = coordinates["chrom"]
    pos = coordinates["pos"]

    # These are integers that will be used when uploading
    found_homozygote = 0
    found_hemizygote = 0

    # Only look at genotypes for the present individuals
    if sv:
        found_variant = True
    else:
        found_variant = False
        for ind_obj in case_obj["individuals"]:
            # Get the index position for the individual in the VCF
            ind_pos = ind_obj["ind_index"]

            if gq_qual:
                gq = -1
                if variant.QUAL:
                    gq = int(variant.QUAL)

            if not gq_qual:
                gq = int(variant.gt_quals[ind_pos])

            # When gq is missing in FORMAT cyvcf2 assigns a score of -1
            if (gq_threshold and 0 <= gq < gq_threshold) or (gq == -1 and not ignore_gq_if_unset):
                continue

            genotype = GENOTYPE_MAP[variant.gt_types[ind_pos]]

            if genotype in ["het", "hom_alt"]:
                LOG.debug("Found variant")
                found_variant = True

                # If variant in X or Y and individual is male,
                # we need to check hemizygosity
                if (
                    chrom in ["X", "Y", "chrX", "chrY"]
                    and ind_obj["sex"] == 1
                    and not check_par(chrom, pos, genome_build=genome_build)
                ):
                    LOG.debug("Found hemizygous variant")
                    found_hemizygote = 1

                if genotype == "hom_alt":
                    LOG.debug("Found homozygote alternative variant")
                    found_homozygote = 1

    if found_variant:
        variant_obj = Variant(
            variant_id=variant_id,
            chrom=chrom,
            pos=pos,
            end=coordinates["end"],
            ref=ref,
            alt=alt,
            end_chrom=coordinates["end_chrom"],
            sv_type=coordinates["sv_type"],
            sv_len=coordinates["sv_length"],
            case_id=case_id,
            homozygote=found_homozygote,
            hemizygote=found_hemizygote,
            is_sv=sv,
            id_column=variant.ID,
        )

    return variant_obj



# -*- coding: utf-8 -*-


class Case(dict):
    """Represent a Case."""

    def __init__(
        self,
        case_id,
        vcf_path=None,
        vcfsv_path=None,
        nr_variants=None,
        nr_sv_variants=None,
        profile_path=None,
    ):
        super(Case, self).__init__(
            case_id=case_id,
            vcf_path=vcf_path,
            vcf_sv_path=vcfsv_path,
            nr_variants=nr_variants,
            nr_sv_variants=nr_sv_variants,
            profile_path=profile_path,
        )
        self["individuals"] = []
        self["sv_individuals"] = []
        self["_inds"] = {}
        self["_sv_inds"] = {}


class Individual(dict):
    """Individual representation."""

    def __init__(
        self,
        ind_id,
        case_id=None,
        mother=None,
        father=None,
        sex=None,
        phenotype=None,
        ind_index=None,
        profile=None,
        similar_samples=None,
    ):
        """Construct a individual object

        Args:
            ind_id (str): The individual id
            case_id (str): What case it belongs to
            mother (str): The mother id
            father (str): The father id
            sex (str): Sex in ped format
            phenotype (str): Phenotype in ped format
            ind_index (int): Column in the vcf.
        """
        super(Individual, self).__init__(
            ind_id=ind_id,
            name=ind_id,
            case_id=case_id,
            ind_index=ind_index,
            sex=sex,
        )

        if profile:
            self["profile"] = profile
        if similar_samples:
            self["similar_samples"] = similar_samples



import logging

from pymongo import ASCENDING, IndexModel

logger = logging.getLogger(__name__)

__version__ = "2.7.21"

INDEXES = {
    "variant": [
        IndexModel(
            [
                ("chrom", ASCENDING),
                ("start", ASCENDING),
                ("end", ASCENDING),
            ],
            name="coordinates",
        ),
        IndexModel(
            [
                ("chrom", ASCENDING),
                ("end", ASCENDING),
            ],
            name="end",
        ),
    ],
    "structural_variant": [
        IndexModel(
            [
                ("sv_type", ASCENDING),
                ("chrom", ASCENDING),
                ("end_chrom", ASCENDING),
                ("pos_left", ASCENDING),
                ("pos_right", ASCENDING),
            ],
            name="coordinates",
            background=True,
        ),
        IndexModel(
            [
                ("chrom", ASCENDING),
                ("pos_left", ASCENDING),
                ("end_right", ASCENDING),
            ],
            name="short_coordinates",
            background=True,
        ),
    ],
    "identity": [
        IndexModel(
            [
                ("cluster_id", ASCENDING),
            ],
            name="cluster",
        ),
        IndexModel(
            [
                ("variant_id", ASCENDING),
            ],
            name="variant",
        ),
    ],
}



import logging
import os

from cyvcf2 import VCF
from loqusdb.build_models.variant import get_variant_id
from loqusdb.exceptions import VcfError

LOG = logging.getLogger(__name__)
VALID_ENDINGS = [".vcf", ".gz", ".bcf"]


def add_headers(vcf_obj, nr_cases=None, sv=False):
    """Add loqus specific information to a VCF header

    Args:
        vcf_obj(cyvcf2.VCF)
    """

    vcf_obj.add_info_to_header(
        {
            "ID": "Obs",
            "Number": "1",
            "Type": "Integer",
            "Description": "The number of observations for the variant",
        }
    )
    if not sv:
        vcf_obj.add_info_to_header(
            {
                "ID": "Hom",
                "Number": "1",
                "Type": "Integer",
                "Description": "The number of observed homozygotes",
            }
        )
        vcf_obj.add_info_to_header(
            {
                "ID": "Hem",
                "Number": "1",
                "Type": "Integer",
                "Description": "The number of observed hemizygotes",
            }
        )
    if nr_cases:
        case_header = "##NrCases={}".format(nr_cases)
        vcf_obj.add_to_header(case_header)
    # head.add_version_tracking("loqusdb", version, datetime.now().strftime("%Y-%m-%d %H:%M"))
    return


def get_file_handle(file_path):
    """Return cyvcf2 VCF object

    Args:
        file_path(str)

    Returns:
        vcf_obj(cyvcf2.VCF)
    """
    LOG.debug("Check if file end is correct")

    if not os.path.exists(file_path):
        raise IOError("No such file:{0}".format(file_path))

    if os.path.splitext(file_path)[-1] not in VALID_ENDINGS:
        raise IOError("Not a valid vcf file name: {}".format(file_path))

    return VCF(file_path)


def get_vcf(file_path):
    """Yield variants from a vcf file

    Args:
        file_path(str)

    Yields:
        vcf_obj(cyvcf2.VCF): An iterable with cyvcf2.Variant

    """

    vcf_obj = get_file_handle(file_path)

    return vcf_obj


def check_sorting(previous_chrom, previous_pos, current_chrom, current_pos):
    """docstring for check_sorting"""
    pass


def check_vcf(vcf_path, keep_chr_prefix=None, expected_type="snv"):
    """Check if there are any problems with the vcf file

    Args:
        vcf_path(str)
        expected_type(str): 'sv' or 'snv'

    Returns:
        vcf_info(dict): dict like
        {
            'nr_variants':<INT>,
            'variant_type': <STR> in ['snv', 'sv'],
            'individuals': <LIST> individual positions in file
        }
    """
    LOG.info("Check if vcf is on correct format...")

    vcf = VCF(vcf_path)
    individuals = vcf.samples
    variant_type = None

    previous_pos = None
    previous_chrom = None

    position_variants = set()

    nr_variants = 0
    for nr_variants, variant in enumerate(vcf, 1):

        # Check the type of variant
        current_type = "sv" if variant.var_type == "sv" else "snv"
        if not variant_type:
            variant_type = current_type

        # Vcf can not include both snvs and svs
        if variant_type != current_type:
            raise VcfError("Vcf includes a mix of snvs and svs")

        current_chrom = variant.CHROM
        current_pos = variant.POS

        # We start with a simple id that can be used by SV:s
        variant_id = "{0}_{1}".format(current_chrom, current_pos)
        # For SNVs we can create a proper variant id with chrom_pos_ref_alt
        if variant_type == "snv":
            variant_id = get_variant_id(variant, keep_chr_prefix)

        # Initiate variables
        if not previous_chrom:
            previous_chrom = current_chrom
            previous_pos = current_pos
            position_variants = {variant_id}
            continue

        # Update variables if new chromosome
        if current_chrom != previous_chrom:
            previous_chrom = current_chrom
            previous_pos = current_pos
            position_variants = {variant_id}
            continue

        if variant_type == "snv":
            # Check if variant is unique
            if current_pos == previous_pos:
                if variant_id in position_variants:
                    raise VcfError("Variant {0} occurs several times" " in vcf".format(variant_id))
                else:
                    position_variants.add(variant_id)
            # Check if vcf is sorted
            else:
                if not current_pos >= previous_pos:
                    raise VcfError("Vcf if not sorted in a correct way")
                previous_pos = current_pos
                # Reset position_variants since we are on a new position
                position_variants = {variant_id}

    if variant_type != expected_type:
        raise VcfError(
            "VCF file does not only include {0}s, please check vcf {1}".format(
                expected_type.upper(), vcf_path
            )
        )

    LOG.info("Vcf file %s looks fine", vcf_path)
    LOG.info("Nr of variants in vcf: {0}".format(nr_variants))
    LOG.info("Type of variants in vcf: {0}".format(variant_type))

    return {
        "nr_variants": nr_variants,
        "variant_type": variant_type,
        "individuals": individuals,
    }



# -*- coding: utf-8 -*-
import logging
from datetime import datetime

import click
from loqusdb.exceptions import CaseError
from loqusdb.utils.case import get_case
from loqusdb.utils.delete import delete as delete_command

from loqusdb.commands.cli import cli as base_command

LOG = logging.getLogger(__name__)


@base_command.command("delete", short_help="Delete the variants of a family")
@click.option("-f", "--family-file", type=click.Path(exists=True), metavar="<ped_file>")
@click.option(
    "-t",
    "--family-type",
    type=click.Choice(["ped", "alt", "cmms", "mip"]),
    default="ped",
    help="If the analysis use one of the known setups, please specify which one.",
)
@click.option(
    "-c",
    "--case-id",
    type=str,
    help="If a different case id than the one in ped file should be used",
)
@click.pass_context
def delete(ctx, family_file, family_type, case_id):
    """Delete the variants of a case."""
    if not (family_file or case_id):
        LOG.error("Please provide a family file")
        ctx.abort()

    adapter = ctx.obj["adapter"]
    keep_chr_prefix = ctx.obj["keep_chr_prefix"]

    # Get a ped_parser.Family object from family file
    family = None
    family_id = None
    if family_file:
        with open(family_file, "r") as family_lines:
            family = get_case(family_lines=family_lines, family_type=family_type)
            family_id = family.family_id

    # There has to be a case_id or a family at this stage.
    case_id = case_id or family_id

    if not case_id:
        LOG.warning("Please provide a case id")
        ctx.abort()

    existing_case = adapter.case({"case_id": case_id})
    if not existing_case:
        LOG.warning("Case %s does not exist in database" % case_id)
        return

    genome_build = ctx.obj["genome_build"]
    start_deleting = datetime.now()
    try:
        delete_command(
            adapter=adapter,
            case_obj=existing_case,
            genome_build=genome_build,
            keep_chr_prefix=keep_chr_prefix,
        )
    except (CaseError, IOError) as error:
        LOG.warning(error)
        ctx.abort()



from loqusdb.build_models.variant import get_variant_id
from loqusdb.utils.delete import delete_variants
from loqusdb.constants import GRCH37, GRCH38


def test_delete_variants(real_mongo_adapter, het_variant, case_obj):
    ## GIVEN a database with one variant
    db = real_mongo_adapter.db
    case_id = case_obj["case_id"]

    db.variant.insert_one(
        {
            "_id": get_variant_id(het_variant),
            "families": [case_id],
            "observations": 1,
        }
    )

    mongo_variant = db.variant.find_one()
    assert mongo_variant["families"] == [case_id]

    ## WHEN deleting the variant
    delete_variants(
        adapter=real_mongo_adapter, vcf_obj=[het_variant], case_obj=case_obj, genome_build=GRCH37
    )

    mongo_variant = db.variant.find_one()

    ## THEN assert that the variant was not found
    assert mongo_variant is None


def test_delete_variant(real_mongo_adapter, het_variant, case_obj):
    ## GIVEN a database with one variant that is observed twice
    db = real_mongo_adapter.db
    case_id = case_obj["case_id"]

    db.variant.insert_one(
        {
            "_id": get_variant_id(het_variant),
            "families": [case_id, "2"],
            "observations": 2,
        }
    )

    mongo_variant = db.variant.find_one()
    assert mongo_variant["observations"] == 2

    ## WHEN deleting the variant for one case
    delete_variants(
        adapter=real_mongo_adapter,
        vcf_obj=[het_variant],
        case_obj=case_obj,
        case_id="2",
        genome_build=GRCH37,
    )

    mongo_variant = db.variant.find_one()

    ## THEN assert that one case has been removed from 'families'
    assert mongo_variant["families"] == [case_id]
    ## THEN assert that the observation count is decreased
    assert mongo_variant["observations"] == 1


def test_delete_non_existing_variant(mongo_adapter, het_variant, case_obj):
    """docstring for test_load_variants"""
    ## GIVEN a mongo adapter to an empty database
    db = mongo_adapter.db
    case_id = case_obj["case_id"]

    ## WHEN deleting the variants
    delete_variants(
        adapter=mongo_adapter, vcf_obj=[het_variant], case_obj=case_obj, genome_build=GRCH37
    )

    # THEN assert nothing happens
    mongo_variant = db.variant.find_one()

    assert mongo_variant == None


def test_delete_variants_grch38(real_mongo_adapter, het_variant, case_obj):
    ## GIVEN a database with one variant
    db = real_mongo_adapter.db
    case_id = case_obj["case_id"]

    db.variant.insert_one(
        {
            "_id": get_variant_id(het_variant),
            "families": [case_id],
            "observations": 1,
        }
    )

    mongo_variant = db.variant.find_one()
    assert mongo_variant["families"] == [case_id]

    ## WHEN deleting the variant
    delete_variants(
        adapter=real_mongo_adapter, vcf_obj=[het_variant], case_obj=case_obj, genome_build=GRCH38
    )

    mongo_variant = db.variant.find_one()

    ## THEN assert that the variant was not found
    assert mongo_variant is None


def test_delete_variant_grch38(real_mongo_adapter, het_variant, case_obj):
    ## GIVEN a database with one variant that is observed twice
    db = real_mongo_adapter.db
    case_id = case_obj["case_id"]

    db.variant.insert_one(
        {
            "_id": get_variant_id(het_variant),
            "families": [case_id, "2"],
            "observations": 2,
        }
    )

    mongo_variant = db.variant.find_one()
    assert mongo_variant["observations"] == 2

    ## WHEN deleting the variant for one case
    delete_variants(
        adapter=real_mongo_adapter,
        vcf_obj=[het_variant],
        case_obj=case_obj,
        case_id="2",
        genome_build=GRCH38,
    )

    mongo_variant = db.variant.find_one()

    ## THEN assert that one case has been removed from 'families'
    assert mongo_variant["families"] == [case_id]
    ## THEN assert that the observation count is decreased
    assert mongo_variant["observations"] == 1


def test_delete_non_existing_variant_grch38(mongo_adapter, het_variant, case_obj):
    """docstring for test_load_variants"""
    ## GIVEN a mongo adapter to an empty database
    db = mongo_adapter.db
    case_id = case_obj["case_id"]

    ## WHEN deleting the variants
    delete_variants(
        adapter=mongo_adapter, vcf_obj=[het_variant], case_obj=case_obj, genome_build=GRCH38
    )

    # THEN assert nothing happens
    mongo_variant = db.variant.find_one()

    assert mongo_variant == None



from mongomock import MongoClient

from loqusdb.plugins.mongo.adapter import MongoAdapter


class MockFlaskApp(object):
    """Mock a Flask App"""

    def __init__(self, host="localhost", port=27017, db_name="test"):
        client = MongoClient()

        self.config = {}
        self.extensions = {"pymongo": {}}

        self.config["MONGO_DBNAME"] = db_name
        self.config["MONGO_PORT"] = port
        self.config["MONGO_HOST"] = host

        # This is how flaskpymongo sets it up:
        self.extensions["pymongo"]["MONGO"] = [client, client[db_name]]


def test_init_app(mongo_client):
    app = MockFlaskApp()
    adapter = MongoAdapter()
    adapter.init_app(app)
    assert adapter.db_name == app.config["MONGO_DBNAME"]



class ProfileVariant(dict):
    """Represent a profile variant

    These variants will be used to profile samples, based on a samples genotype
    at these positions.
    """

    def __init__(self, variant_id, chrom, pos, ref, alt, maf, id_column=None):
        super(ProfileVariant, self).__init__(
            _id=variant_id, chrom=chrom, pos=pos, ref=ref, alt=alt, maf=maf, id_column=id_column
        )



from loqusdb.build_models.variant import Position, is_greater
from loqusdb.constants import GRCH37, GRCH38


def test_is_greater_different_chrom():
    ## GIVEN two positions where a is smaller than b. Different chroms
    a = Position("1", 100)
    b = Position("3", 100)

    ## WHEN testing if a is greater than b
    res = is_greater(a, b, genome_build=GRCH37)
    ## THEN assert a was not greater than b
    assert res is False

    ## WHEN testing if b is greater than a
    res = is_greater(b, a, genome_build=GRCH37)
    ## THEN assert b was greater than a
    assert res is True


def test_is_greater_same_chrom():
    ## GIVEN two positions where a is smaller than b. Same chroms
    a = Position("1", 100)
    b = Position("1", 300)

    ## WHEN testing if a is greater than b
    res = is_greater(a, b, genome_build=GRCH37)
    ## THEN assert a was not greater than b
    assert res is False

    ## WHEN testing if b is greater than a
    res = is_greater(b, a, genome_build=GRCH37)
    ## THEN assert b was greater than a
    assert res is True


def test_is_greater_wierd_chrom():
    ## GIVEN two positions where one chrom is unknown b
    a = Position("1", 100)
    b = Position("GRLrs2", 300)

    ## WHEN testing if a is greater than b
    res = is_greater(a, b, genome_build=GRCH37)
    ## THEN assert a was not greater than b
    assert res is False

    ## WHEN testing if b is greater than a
    res = is_greater(b, a, genome_build=GRCH37)
    ## THEN assert a was not greater than b
    assert res is False


def test_is_greater_different_chrom_grch38():
    ## GIVEN two positions where a is smaller than b. Different chroms
    a = Position("chr1", 100)
    b = Position("chr3", 100)

    ## WHEN testing if a is greater than b
    res = is_greater(a, b, genome_build=GRCH38)
    ## THEN assert a was not greater than b
    assert res is False

    ## WHEN testing if b is greater than a
    res = is_greater(b, a, genome_build=GRCH38)
    ## THEN assert b was greater than a
    assert res is True


def test_is_greater_same_chrom_grch38():
    ## GIVEN two positions where a is smaller than b. Same chroms
    a = Position("chr1", 100)
    b = Position("chr1", 300)

    ## WHEN testing if a is greater than b
    res = is_greater(a, b, genome_build=GRCH38)
    ## THEN assert a was not greater than b
    assert res is False

    ## WHEN testing if b is greater than a
    res = is_greater(b, a, genome_build=GRCH38)
    ## THEN assert b was greater than a
    assert res is True


def test_is_greater_wierd_chrom_grch38():
    ## GIVEN two positions where one chrom is unknown b
    a = Position("chr1", 100)
    b = Position("GRLrs2", 300)

    ## WHEN testing if a is greater than b
    res = is_greater(a, b, genome_build=GRCH38)
    ## THEN assert a was not greater than b
    assert res is False

    ## WHEN testing if b is greater than a
    res = is_greater(b, a, genome_build=GRCH38)
    ## THEN assert a was not greater than b
    assert res is False



from loqusdb.build_models.variant import build_variant
from loqusdb.constants import GRCH37, GRCH38


def test_load_insertion(small_insert_variant, mongo_adapter, case_obj):
    adapter = mongo_adapter
    ## GIVEN a mongo adapter with a case
    variant = small_insert_variant
    case_id = case_obj["case_id"]
    adapter.add_case(case_obj)

    ## WHEN loading a small insertion
    formated_variant = build_variant(
        variant=variant, case_obj=case_obj, case_id=case_id, genome_build=GRCH37
    )
    adapter.add_structural_variant(formated_variant)

    ## THEN assert the object returned is correct
    variant_cluster = adapter.db.structural_variant.find_one()

    assert variant_cluster["families"] == [case_id]


def test_load_same_insertion_twice(small_insert_variant, mongo_adapter, case_obj):
    adapter = mongo_adapter
    ## GIVEN a mongo adapter with a case
    variant = small_insert_variant
    case_id = case_obj["case_id"]
    adapter.add_case(case_obj)

    ## WHEN loading a small insertion
    formated_variant = build_variant(
        variant=variant, case_obj=case_obj, case_id=case_id, genome_build=GRCH37
    )
    adapter.add_structural_variant(formated_variant)
    formated_variant["case_id"] = "2"
    adapter.add_structural_variant(formated_variant)

    ## THEN assert the object returned is correct
    variant_cluster = adapter.db.structural_variant.find_one()

    assert set(variant_cluster["families"]) == set([case_id, "2"])


def test_load_translocation(translocation_variant, case_obj, mongo_adapter):
    adapter = mongo_adapter
    ## GIVEN a mongo adapter with a case
    variant = translocation_variant
    case_id = case_obj["case_id"]
    adapter.add_case(case_obj)

    ## WHEN loading a small insertion
    formated_variant = build_variant(
        variant=variant, case_obj=case_obj, case_id=case_id, genome_build=GRCH37
    )
    adapter.add_structural_variant(formated_variant)

    ## THEN assert the object returned is correct
    variant_cluster = adapter.db.structural_variant.find_one()

    assert variant_cluster["families"] == [case_id]


def test_load_same_translocation_twice(translocation_variant, case_obj, mongo_adapter):
    adapter = mongo_adapter
    ## GIVEN a mongo adapter with a case
    variant = translocation_variant
    case_id = case_obj["case_id"]
    adapter.add_case(case_obj)

    ## WHEN loading a small insertion
    formated_variant = build_variant(
        variant=variant, case_obj=case_obj, case_id=case_id, genome_build=GRCH37
    )
    adapter.add_structural_variant(formated_variant)

    formated_variant["case_id"] = "2"
    adapter.add_structural_variant(formated_variant)

    ## THEN assert the object returned is correct
    variant_cluster = adapter.db.structural_variant.find_one()

    assert set(variant_cluster["families"]) == set([case_id, "2"])


def test_load_insertion_grch38(chr_small_insert_variant, mongo_adapter, case_obj):
    adapter = mongo_adapter
    ## GIVEN a mongo adapter with a case
    variant = chr_small_insert_variant
    case_id = case_obj["case_id"]
    adapter.add_case(case_obj)

    ## WHEN loading a small insertion
    formated_variant = build_variant(
        variant=variant, case_obj=case_obj, case_id=case_id, genome_build=GRCH37
    )
    adapter.add_structural_variant(formated_variant)

    ## THEN assert the object returned is correct
    variant_cluster = adapter.db.structural_variant.find_one()

    assert variant_cluster["families"] == [case_id]


def test_load_same_insertion_twice_grch38(chr_small_insert_variant, mongo_adapter, case_obj):
    adapter = mongo_adapter
    ## GIVEN a mongo adapter with a case
    variant = chr_small_insert_variant
    case_id = case_obj["case_id"]
    adapter.add_case(case_obj)

    ## WHEN loading a small insertion
    formated_variant = build_variant(
        variant=variant, case_obj=case_obj, case_id=case_id, genome_build=GRCH37
    )
    adapter.add_structural_variant(formated_variant)
    formated_variant["case_id"] = "2"
    adapter.add_structural_variant(formated_variant)

    ## THEN assert the object returned is correct
    variant_cluster = adapter.db.structural_variant.find_one()

    assert set(variant_cluster["families"]) == set([case_id, "2"])


def test_load_translocation_grch38(chr_translocation_variant, case_obj, mongo_adapter):
    adapter = mongo_adapter
    ## GIVEN a mongo adapter with a case
    variant = chr_translocation_variant
    case_id = case_obj["case_id"]
    adapter.add_case(case_obj)

    ## WHEN loading a small insertion
    formated_variant = build_variant(
        variant=variant, case_obj=case_obj, case_id=case_id, genome_build=GRCH37
    )
    adapter.add_structural_variant(formated_variant)

    ## THEN assert the object returned is correct
    variant_cluster = adapter.db.structural_variant.find_one()

    assert variant_cluster["families"] == [case_id]


def test_load_same_translocation_twice_grch38(chr_translocation_variant, case_obj, mongo_adapter):
    adapter = mongo_adapter
    ## GIVEN a mongo adapter with a case
    variant = chr_translocation_variant
    case_id = case_obj["case_id"]
    adapter.add_case(case_obj)

    ## WHEN loading a small insertion
    formated_variant = build_variant(
        variant=variant, case_obj=case_obj, case_id=case_id, genome_build=GRCH37
    )
    adapter.add_structural_variant(formated_variant)

    formated_variant["case_id"] = "2"
    adapter.add_structural_variant(formated_variant)

    ## THEN assert the object returned is correct
    variant_cluster = adapter.db.structural_variant.find_one()

    assert set(variant_cluster["families"]) == set([case_id, "2"])



import logging

from click import progressbar

LOG = logging.getLogger(__name__)


def migrate_database(adapter):
    """Migrate an old loqusdb instance to 1.0

    Args:
        adapter

    Returns:
        nr_updated(int): Number of variants that where updated
    """

    all_variants = adapter.get_variants()
    nr_variants = adapter.nr_variants()
    nr_updated = 0
    with progressbar(all_variants, label="Updating variants", length=nr_variants) as bar:
        for variant in bar:
            # Do not update if the variants have the correct format
            if "chrom" in variant:
                continue
            nr_updated += 1
            splitted_id = variant["_id"].split("_")

            chrom = splitted_id[0]
            start = int(splitted_id[1])
            ref = splitted_id[2]
            alt = splitted_id[3]

            # Calculate end
            end = start + (max(len(ref), len(alt)) - 1)

            adapter.db.variant.find_one_and_update(
                {"_id": variant["_id"]}, {"$set": {"chrom": chrom, "start": start, "end": end}}
            )

    return nr_updated



from click.testing import CliRunner

from loqusdb.commands.cli import cli as base_command
from loqusdb.exceptions import ProfileError


def test_base_command():
    runner = CliRunner()
    result = runner.invoke(base_command, ["--help"])

    assert result.exit_code == 0


def test_load_command(vcf_path, ped_path, real_mongo_adapter, real_db_name):
    ## GIVEN a vcf_path a ped_path and a empty database
    assert sum([1 for case in real_mongo_adapter.cases()]) == 0
    runner = CliRunner()
    ## WHEN inserting a case via the CLI
    command = ["--database", real_db_name, "load", "--variant-file", vcf_path, "-f", ped_path]
    result = runner.invoke(base_command, command)
    ## THEN assert that the cli exits without problems
    assert result.exit_code == 0
    ## THEN assert that the case was added
    assert sum([1 for case in real_mongo_adapter.cases()]) == 1


def test_load_command_no_ped_case_id(vcf_path, case_id, real_mongo_adapter, real_db_name):
    ## GIVEN a vcf_path a ped_path and a empty database
    assert real_mongo_adapter.case({"case_id": case_id}) is None
    runner = CliRunner()
    ## WHEN inserting a case via the CLI
    command = ["--database", real_db_name, "load", "--variant-file", vcf_path, "-c", case_id]
    result = runner.invoke(base_command, command)
    ## THEN assert that the cli exits without problems
    assert result.exit_code == 0
    ## THEN assert that the case was added
    assert real_mongo_adapter.case({"case_id": case_id})


def test_load_command_no_ped(vcf_path, ped_path, real_mongo_adapter, real_db_name):
    ## GIVEN a vcf_path a ped_path and a empty database
    runner = CliRunner()
    ## WHEN inserting a case via the CLI without a ped file
    command = ["--database", real_db_name, "load", "--variant-file", vcf_path]
    result = runner.invoke(base_command, command)

    ## THEN assert that the cli exits with code 1
    assert result.exit_code == 1


def test_load_command_check_profile(
    zipped_vcf_path, ped_path, real_mongo_adapter, real_db_name, profile_vcf_path
):
    NEW_CASE_NAME = "other_case"
    # Load profile variants
    runner = CliRunner()
    command = ["--database", real_db_name, "profile", "--load", "--variant-file", profile_vcf_path]
    result = runner.invoke(base_command, command)

    # WHEN load command with --check-profile flag
    command = [
        "--database",
        real_db_name,
        "load",
        "--variant-file",
        zipped_vcf_path,
        "-f",
        ped_path,
        "--check-profile",
        zipped_vcf_path,
    ]
    result = runner.invoke(base_command, command)
    ## THEN assert that the cli exits without problems
    assert result.exit_code == 0
    ## THEN assert if field 'profile is found in the individual documents'
    for case in real_mongo_adapter.cases():
        for individual in case["individuals"]:
            assert individual.get("profile")

    ##WHEN load the same case (change case-id with -c option)
    command = [
        "--database",
        real_db_name,
        "load",
        "--variant-file",
        zipped_vcf_path,
        "-f",
        ped_path,
        "--check-profile",
        zipped_vcf_path,
        "-c",
        "other_case",
    ]
    result = runner.invoke(base_command, command)
    ## THEN assert that exit_code is not 0 and error is raised
    assert result.exit_code != 0
    assert type(result.exception) == ProfileError

    ##WHEN try to insert the same case without --check-profile flag (should work)
    command = [
        "--database",
        real_db_name,
        "load",
        "--variant-file",
        zipped_vcf_path,
        "-f",
        ped_path,
        "-c",
        NEW_CASE_NAME,
    ]
    result = runner.invoke(base_command, command)
    ## THEN assert that exit_code is 0
    assert result.exit_code == 0
    ## THEN assert that no profile string is added to the individuals
    for individual in real_mongo_adapter.case({"case_id": NEW_CASE_NAME})["individuals"]:
        assert not individual.get("profile")


def test_load_profile_command(profile_vcf_path, real_mongo_adapter, real_db_name):
    runner = CliRunner()

    command = ["--database", real_db_name, "profile", "--load", "--variant-file", profile_vcf_path]
    result = runner.invoke(base_command, command)

    assert result.exit_code == 0
    assert len(list(real_mongo_adapter.profile_variants())) > 0


def test_delete_command_family_file(vcf_path, ped_path, real_mongo_adapter, real_db_name):
    ## GIVEN a vcf_path a ped_path and a empty database
    runner = CliRunner()
    assert sum([1 for case in real_mongo_adapter.cases()]) == 0

    ## WHEN inserting a case via the CLI without a ped file
    load_command = ["--database", real_db_name, "load", "--variant-file", vcf_path, "-f", ped_path]
    result = runner.invoke(base_command, load_command)
    ## THEN assert that the case was added
    assert sum(1 for case in real_mongo_adapter.cases()) == 1

    ## WHEN deleting the case a case via the CLI without a ped file
    delete_command = ["--database", real_db_name, "delete", "-f", ped_path]
    result = runner.invoke(base_command, delete_command)

    ## THEN assert that the cli exits without problems
    assert result.exit_code == 0
    ## THEN assert that the case was deleted
    assert sum(1 for case in real_mongo_adapter.cases()) == 0


def test_delete_command_case_id(vcf_path, case_id, real_mongo_adapter, real_db_name):
    ## GIVEN a vcf_path a ped_path and a empty database
    runner = CliRunner()
    assert real_mongo_adapter.case({"case_id": case_id}) is None

    ## WHEN inserting a case via the CLI without a ped file
    load_command = ["--database", real_db_name, "load", "--variant-file", vcf_path, "-c", case_id]
    result = runner.invoke(base_command, load_command)
    ## THEN assert that the case was added
    assert isinstance(real_mongo_adapter.case({"case_id": case_id}), dict)

    ## WHEN deleting the case a case via the CLI without a ped file
    delete_command = ["--database", real_db_name, "delete", "-c", case_id]
    result = runner.invoke(base_command, delete_command)

    ## THEN assert that the cli exits without problems
    assert result.exit_code == 0
    ## THEN assert that the case was deleted
    assert real_mongo_adapter.case({"case_id": case_id}) is None


def test_cases_command_case_id(vcf_path, case_id, real_mongo_adapter, real_db_name):
    ## GIVEN a vcf_path a ped_path and a empty database
    runner = CliRunner()
    assert real_mongo_adapter.case({"case_id": case_id}) is None

    ## WHEN inserting a case via the CLI without a ped file
    load_command = ["--database", real_db_name, "load", "--variant-file", vcf_path, "-c", case_id]
    result = runner.invoke(base_command, load_command)
    ## THEN assert that the case was added
    assert isinstance(real_mongo_adapter.case({"case_id": case_id}), dict)

    ## WHEN searching for the case with CLI
    command = ["--database", real_db_name, "cases", "--case-id", case_id]
    result = runner.invoke(base_command, command)

    ## THEN assert that the cli exits without problems
    assert result.exit_code == 0


def test_cases_command_non_existing(vcf_path, case_id, real_mongo_adapter, real_db_name):
    ## GIVEN a vcf_path a ped_path and a empty database
    runner = CliRunner()
    assert real_mongo_adapter.case({"case_id": case_id}) is None

    ## WHEN searching for a non existing case
    command = ["--database", real_db_name, "cases", "--case-id", "hello"]
    result = runner.invoke(base_command, command)

    ## THEN assert that the exit is zero
    assert result.exit_code == 0
    # Exit code should be 0 even if there is no result returned


# def test_wipe_command(mongo_client, vcf_path, ped_path):
#     runner = CliRunner()
#
#     load_command = ['-c', 'mongomock://', '--database', 'test', 'load', vcf_path, '-f', ped_path]
#     runner.invoke(base_command, load_command)
#
#     wipe_command = ['-c', 'mongomock://', '--database', 'test', 'wipe', '--yes']
#     result = runner.invoke(base_command, wipe_command)
#
#     assert result.exit_code == 0



from loqusdb.utils.migrate import migrate_database


def test_migrate_one_variant(mongo_adapter):
    ##GIVEN a adapter to an empty old database
    adapter = mongo_adapter
    assert sum(1 for i in adapter.get_variants()) == 0

    ##WHEN inserting a old style variant and updating the database
    chrom = "1"
    pos = 880086
    ref = "T"
    alt = "C"
    hom = hem = 0
    obs = 3
    families = ["recessive_trio", "1", "2"]
    variant = {
        "_id": "_".join([chrom, str(pos), ref, alt]),
        "homozygote": hom,
        "hemizygote": hem,
        "observations": obs,
        "families": families,
    }
    adapter.db.variant.insert_one(variant)
    ##THEN assert that the variant was updated correct

    nr_updated = migrate_database(adapter)
    assert nr_updated == 1

    migrated_variant = adapter.db.variant.find_one()

    assert migrated_variant["chrom"] == chrom
    assert migrated_variant["start"] == pos
    assert migrated_variant["end"] == pos
    assert migrated_variant["homozygote"] == hom
    assert migrated_variant["hemizygote"] == hem
    assert migrated_variant["observations"] == obs
    assert migrated_variant["families"] == families



from loqusdb.build_models.variant import build_variant
from loqusdb.constants import GRCH37, GRCH38


def test_format_indel(del_variant, case_obj):
    ## GIVEN a SV deletion
    variant = del_variant
    case_id = case_obj["case_id"]
    ## WHEN parsing the variant
    formated_variant = build_variant(
        variant=variant, case_obj=case_obj, case_id=case_id, genome_build=GRCH37
    )
    expected_id = "_".join([variant.CHROM, str(variant.POS), variant.REF, variant.ALT[0]])

    ## THEN assert the sv is parsed correct
    assert formated_variant
    assert formated_variant["variant_id"] == expected_id
    assert formated_variant["chrom"] == variant.CHROM
    assert formated_variant["end_chrom"] == variant.CHROM
    assert formated_variant["pos"] == variant.POS
    assert formated_variant["end"] == variant.INFO["END"]
    assert formated_variant["sv_len"] == abs(variant.INFO["SVLEN"])

    assert formated_variant["ref"] == variant.REF
    assert formated_variant["alt"] == variant.ALT[0]
    assert formated_variant["sv_type"] == "DEL"
    assert formated_variant["case_id"] == case_id
    assert formated_variant["homozygote"] == 0
    assert formated_variant["hemizygote"] == 0


def test_format_indel_chrprefix(chr_del_variant, case_obj):
    ## GIVEN a SV deletion
    variant = chr_del_variant
    case_id = case_obj["case_id"]
    ## WHEN parsing the variant
    formated_variant = build_variant(
        variant=variant,
        case_obj=case_obj,
        case_id=case_id,
        genome_build=GRCH38,
        keep_chr_prefix=True,
    )
    expected_id = "_".join([variant.CHROM, str(variant.POS), variant.REF, variant.ALT[0]])

    ## THEN assert the sv is parsed correct
    assert formated_variant
    assert formated_variant["variant_id"] == expected_id
    assert formated_variant["chrom"] == variant.CHROM
    assert formated_variant["end_chrom"] == variant.CHROM
    assert formated_variant["pos"] == variant.POS
    assert formated_variant["end"] == variant.INFO["END"]
    assert formated_variant["sv_len"] == abs(variant.INFO["SVLEN"])

    assert formated_variant["ref"] == variant.REF
    assert formated_variant["alt"] == variant.ALT[0]
    assert formated_variant["sv_type"] == "DEL"
    assert formated_variant["case_id"] == case_id
    assert formated_variant["homozygote"] == 0
    assert formated_variant["hemizygote"] == 0


def test_format_small_ins(small_insert_variant, case_obj):
    ## GIVEN a small insertion (This means that the insertion is included in ALT field)
    variant = small_insert_variant
    case_id = case_obj["case_id"]
    ## WHEN parsing the variant
    formated_variant = build_variant(
        variant=variant, case_obj=case_obj, case_id=case_id, genome_build=GRCH37
    )

    ## THEN assert the sv is parsed correct
    assert formated_variant["chrom"] == variant.CHROM
    assert formated_variant["end_chrom"] == variant.CHROM
    assert formated_variant["pos"] == variant.POS
    assert formated_variant["end"] == variant.POS + abs(variant.INFO["SVLEN"])
    assert formated_variant["sv_len"] == abs(variant.INFO["SVLEN"])

    assert formated_variant["ref"] == variant.REF
    assert formated_variant["alt"] == variant.ALT[0]
    assert formated_variant["sv_type"] == "INS"


def test_format_small_ins_chrprefix(chr_small_insert_variant, case_obj):
    ## GIVEN a small insertion (This means that the insertion is included in ALT field)
    variant = chr_small_insert_variant
    case_id = case_obj["case_id"]
    ## WHEN parsing the variant
    formated_variant = build_variant(
        variant=variant,
        case_obj=case_obj,
        case_id=case_id,
        genome_build=GRCH38,
        keep_chr_prefix=True,
    )

    ## THEN assert the sv is parsed correct
    assert formated_variant["chrom"] == variant.CHROM
    assert formated_variant["end_chrom"] == variant.CHROM
    assert formated_variant["pos"] == variant.POS
    assert formated_variant["end"] == variant.POS + abs(variant.INFO["SVLEN"])
    assert formated_variant["sv_len"] == abs(variant.INFO["SVLEN"])

    assert formated_variant["ref"] == variant.REF
    assert formated_variant["alt"] == variant.ALT[0]
    assert formated_variant["sv_type"] == "INS"


def test_format_insertion(insertion_variant, case_obj):
    ## GIVEN a small insertion (This means that the insertion is included in ALT field)
    variant = insertion_variant
    case_id = case_obj["case_id"]
    ## WHEN parsing the variant
    formated_variant = build_variant(
        variant=variant, case_obj=case_obj, case_id=case_id, genome_build=GRCH37
    )

    ## THEN assert the sv is parsed correct
    assert formated_variant["chrom"] == variant.CHROM
    assert formated_variant["end_chrom"] == variant.CHROM
    assert formated_variant["pos"] == variant.POS
    assert formated_variant["end"] == variant.INFO["END"]
    assert formated_variant["sv_len"] == 0

    assert formated_variant["ref"] == variant.REF
    assert formated_variant["alt"] == variant.ALT[0]
    assert formated_variant["sv_type"] == "INS"


def test_format_insertion_chrprefix(chr_insertion_variant, case_obj):
    ## GIVEN a small insertion (This means that the insertion is included in ALT field)
    variant = chr_insertion_variant
    case_id = case_obj["case_id"]
    ## WHEN parsing the variant
    formated_variant = build_variant(
        variant=variant,
        case_obj=case_obj,
        case_id=case_id,
        genome_build=GRCH38,
        keep_chr_prefix=True,
    )

    ## THEN assert the sv is parsed correct
    assert formated_variant["chrom"] == variant.CHROM
    assert formated_variant["end_chrom"] == variant.CHROM
    assert formated_variant["pos"] == variant.POS
    assert formated_variant["end"] == variant.INFO["END"]
    assert formated_variant["sv_len"] == 0

    assert formated_variant["ref"] == variant.REF
    assert formated_variant["alt"] == variant.ALT[0]
    assert formated_variant["sv_type"] == "INS"


def test_format_dup_tandem(duptandem_variant, case_obj):
    ## GIVEN a small insertion (This means that the insertion is included in ALT field)
    variant = duptandem_variant
    case_id = case_obj["case_id"]
    ## WHEN parsing the variant
    formated_variant = build_variant(
        variant=variant, case_obj=case_obj, case_id=case_id, genome_build=GRCH37
    )

    ## THEN assert the sv is parsed correct
    assert formated_variant["chrom"] == variant.CHROM
    assert formated_variant["end_chrom"] == variant.CHROM
    assert formated_variant["pos"] == variant.POS
    assert formated_variant["end"] == variant.INFO["END"]
    assert formated_variant["sv_len"] == abs(variant.INFO["SVLEN"])

    assert formated_variant["ref"] == variant.REF
    assert formated_variant["alt"] == variant.ALT[0]
    assert formated_variant["sv_type"] == "DUP"


def test_format_dup_tandem_chrprefix(chr_duptandem_variant, case_obj):
    ## GIVEN a small insertion (This means that the insertion is included in ALT field)
    variant = chr_duptandem_variant
    case_id = case_obj["case_id"]
    ## WHEN parsing the variant
    formated_variant = build_variant(
        variant=variant,
        case_obj=case_obj,
        case_id=case_id,
        genome_build=GRCH38,
        keep_chr_prefix=True,
    )

    ## THEN assert the sv is parsed correct
    assert formated_variant["chrom"] == variant.CHROM
    assert formated_variant["end_chrom"] == variant.CHROM
    assert formated_variant["pos"] == variant.POS
    assert formated_variant["end"] == variant.INFO["END"]
    assert formated_variant["sv_len"] == abs(variant.INFO["SVLEN"])

    assert formated_variant["ref"] == variant.REF
    assert formated_variant["alt"] == variant.ALT[0]
    assert formated_variant["sv_type"] == "DUP"


def test_format_translocation(translocation_variant, case_obj):
    ## GIVEN a small insertion (This means that the insertion is included in ALT field)
    variant = translocation_variant
    case_id = case_obj["case_id"]
    ## WHEN parsing the variant
    formated_variant = build_variant(
        variant=variant, case_obj=case_obj, case_id=case_id, genome_build=GRCH37
    )

    ## THEN assert the sv is parsed correct
    assert formated_variant["chrom"] == variant.CHROM
    assert formated_variant["end_chrom"] == "11"
    assert formated_variant["pos"] == variant.POS
    assert formated_variant["end"] == 119123896
    assert formated_variant["sv_len"] == float("inf")

    assert formated_variant["ref"] == variant.REF
    assert formated_variant["alt"] == variant.ALT[0]
    assert formated_variant["sv_type"] == "BND"


def test_format_translocation_chrprefix(chr_translocation_variant, case_obj):
    ## GIVEN a small insertion (This means that the insertion is included in ALT field)
    variant = chr_translocation_variant
    case_id = case_obj["case_id"]
    ## WHEN parsing the variant
    formated_variant = build_variant(
        variant=variant,
        case_obj=case_obj,
        case_id=case_id,
        genome_build=GRCH38,
        keep_chr_prefix=True,
    )

    ## THEN assert the sv is parsed correct
    assert formated_variant["chrom"] == variant.CHROM
    assert formated_variant["end_chrom"] == "11"
    assert formated_variant["pos"] == variant.POS
    assert formated_variant["end"] == 119123896
    assert formated_variant["sv_len"] == float("inf")

    assert formated_variant["ref"] == variant.REF
    assert formated_variant["alt"] == variant.ALT[0]
    assert formated_variant["sv_type"] == "BND"



# -*- coding: utf-8 -*-
import logging
import os
import subprocess
from datetime import datetime

import click
from loqusdb.resources import background_path

from loqusdb.commands.cli import cli as base_command

LOG = logging.getLogger(__name__)


@base_command.command("restore", short_help="Restore database from dump")
@click.option(
    "-f",
    "--filename",
    help="If custom named file is to be used",
    type=click.Path(exists=True),
)
@click.pass_context
def restore(ctx, filename):
    """Restore the database from a zipped file.

    Default is to restore from db dump in loqusdb/resources/
    """
    filename = filename or background_path
    if not os.path.isfile(filename):
        LOG.warning("File {} does not exist. Please point to a valid file".format(filename))
        ctx.abort()

    call = ["mongorestore", "--gzip", "--archive={}".format(filename), "--db", ctx.obj.get("db")]
    if ctx.obj.get("uri"):  # if db URI is available use it
        call.append(f"--uri={ctx.obj['uri']}")
    else:  # Otherwise use host and port
        call.append(f"--host={ctx.obj['host']}:{ctx.obj['port']}")

    LOG.info("Restoring database from %s", filename)
    start_time = datetime.now()
    try:
        completed = subprocess.run(call, check=True)
    except subprocess.CalledProcessError as err:
        LOG.warning(err)
        ctx.abort()

    LOG.info("Database restored succesfully")
    LOG.info("Time to restore database: {0}".format(datetime.now() - start_time))



from loqusdb.build_models.variant import get_coords, build_variant
from loqusdb.constants import GRCH37, GRCH38


def test_build_het_variant(het_variant, case_obj):
    variant_obj = build_variant(variant=het_variant, case_obj=case_obj, genome_build=GRCH37)
    assert variant_obj["chrom"] == het_variant.CHROM
    assert variant_obj["homozygote"] == 0
    assert variant_obj["hemizygote"] == 0


def test_get_coords_for_BND(bnd_variant):
    coords = get_coords(bnd_variant, True, GRCH37)
    assert coords["pos"] == coords["end"]
    assert coords["sv_length"] == float("inf")
    assert coords["sv_type"] == "BND"


def test_build_het_variant_grch38(het_variant, case_obj):
    variant_obj = build_variant(variant=het_variant, case_obj=case_obj, genome_build=GRCH38)
    assert variant_obj["chrom"] == het_variant.CHROM
    assert variant_obj["homozygote"] == 0
    assert variant_obj["hemizygote"] == 0


def test_get_coords_for_BND_grch38(bnd_variant):
    coords = get_coords(bnd_variant, True, GRCH38)
    assert coords["pos"] == coords["end"]
    assert coords["sv_length"] == float("inf")
    assert coords["sv_type"] == "BND"
